#include "backend"

bool Backend::connect() {
    // Assume the backend is dead
    islive = false;

    debugmsg ("Connecting to back end " + description() + "\n");
    
    // Create client socket
    if ( (clsocket = socket (PF_INET, SOCK_STREAM, 0)) < 0 )
	throw static_cast<Error>("Failed to create client socket: ") +
	    strerror(errno);

    // Resolve hostname, prepare binding
    struct hostent *hostaddr;
    if (! (hostaddr = gethostbyname(bdef.server().c_str())) ) {
	socketclose (clsocket);
	throw static_cast<Error>("Failed to resolve backend host '") +
	    bdef.server();
    }
    struct sockaddr_in backendaddr;
    backendaddr.sin_family = AF_INET;
    backendaddr.sin_port = htons(bdef.port());
    memcpy ((char *) &backendaddr.sin_addr.s_addr,
	    hostaddr->h_addr_list[0], hostaddr->h_length);

    // Client socket goes into nonblocking mode, so we can connect
    // and enforce a timeout later.
    int flags;
    if ( (flags = fcntl (clsocket, F_GETFL, 0)) == -1 ) {
	socketclose (clsocket);
	throw static_cast<Error>("Failed to get fd flags: ") + strerror(errno);
    }    
    if (fcntl (clsocket, F_SETFL, flags | O_NONBLOCK) == -1) {
	socketclose (clsocket);
	throw static_cast<Error>("Failed to fd in nonblocking mode: ") +
	    strerror(errno);
    }

    // Do the connect
    int conres = ::connect (clsocket, (struct sockaddr *)&backendaddr,
			 sizeof(backendaddr));
    int conerrno = errno;

    // Put socket again in blocking mode.
    if (fcntl (clsocket, F_SETFL, flags) == -1) {
	socketclose (clsocket);
	throw static_cast<Error>("Failed to put fd in blocking mode: ") +
	    strerror(errno);
    }
    
    // Check on the outcome of the connect
    if (!conres || conerrno == EINPROGRESS) {
	// Wait for socket to go writable.
	Fdset fdset (config.backend_timeout());
	fdset.add (clsocket);
	if (fdset.readwriteable() == -1 && fdset.writeable() == clsocket)
	    islive = true;
    }

    if (config.debug()) {
	ostringstream o;
	o << "Back end " << description() << " is ";
	if (islive)
	    o << "alive on socket " << clsocket;
	else
	    o << "NOT ALIVE";
	debugmsg (o.str() + "\n");
    }
    
    return (islive);
}
