#include "sys"
#include "fdset/fdset"
#include "balancer/balancer"

void fdwrite (int fd, int timeout, char const *buf, unsigned buflen) {
    if (config.debug()) {
	ostringstream o;
	o << "About to write " << buflen << " bytes to fd " << fd
	  << ", timeout " << timeout << "\n";
	debugmsg (o.str());
    }

    // Log to dump directory if requested
    if (config.dumpdir().length()) {
	ostringstream of;
	of << config.dumpdir() << "/" << balancer.requestnr() << "." << fd;
	FILE *f;
	if ( (!(f = fopen (of.str().c_str(), "a"))) &&
	     (!(f = fopen (of.str().c_str(), "w"))) )
	    warnmsg ("Cannot write traffic log " + of.str() + ": " +
		     strerror(errno) + "\n");
	else {
	    fwrite (buf, 1, buflen, f);
	    fclose (f);
	}
    }

    // Send to the socket
    unsigned totwritten = 0;
    while (totwritten < buflen) {
	// Wait for the socket to become writeable.
	if (timeout) {
	    Fdset set (timeout);
	    set.add (fd);
	    if (set.writeable() != fd)
		throw static_cast<Error>("Fd ") + fd +
		    " failed to become writable within " + timeout + " sec";
	}

	// Push bytes
	ssize_t nwritten;
#	ifdef SunOS
	if (fd < 3)
	    nwritten = write (fd, buf + totwritten,  buflen - totwritten);
	else
	    nwritten = send (fd, buf + totwritten, buflen - totwritten, 0);
#	else
	nwritten = write (fd, buf + totwritten, buflen - totwritten);
#	endif

	if (config.debug()) {
	    ostringstream o;
	    o << "Sent " << nwritten << " bytes to fd " << fd << "\n";
	    debugmsg (o.str());
	}

	// EINVAL / EINPROGRESS errors are handled as: retry
	// If any bytes were written, we're ok
	if (nwritten >= 1)
	    totwritten += nwritten;
	else if (errno != EINVAL && errno != EINPROGRESS)
	    throw static_cast<Error>("Write/send failed: errno=") +
		errno + ", " +  strerror(errno) + ", result=" + nwritten;
    }
}

