
#include "sys/sys"
#include "config/config"
#include "balancer/balancer"
#include "error/error"

using namespace std;

Config config;
Balancer balancer;

static void sigcatcher (int sig) {
    if (sig == SIGHUP)
	balancer.report(true);
    else if (sig != SIGPIPE)
	balancer.terminate(true);
}

int main (int argc, char **argv) {
    static int relevant_sig[] = {
	SIGHUP, SIGINT, SIGQUIT, SIGABRT, SIGTERM, SIGSTOP, SIGPIPE
    };
    
    try {
	// Load configuration from the commandline, promote verbosity
	config.parsecmdline (argc, argv);

	ostringstream o;
	o << getpid();
	msg ("XR running as PID " + o.str() + "\n");
	
	// Load the signal handler.
	for (unsigned i = 0; i < sizeof(relevant_sig) / sizeof(int); i++)
	    signal (relevant_sig[i], sigcatcher);
	
	// Configure the balancer and start serving.
	balancer.init();
	balancer.serve();
    } catch (Error const &e) {
	cerr << e.what() << endl;
	return (1);
    } catch (...) {
	cerr << "ERROR: Unidentified exception caught" << endl;
    }
    
    return (0);
}
